@@ -19,6 +19,7 @@ from mindsdb_native.libs.constants.mindsdb import (MODEL_STATUS_TRAINED,
                                                    TRANSACTION_ANALYSE)
 from mindsdb_native.libs.helpers.locking import MDBLock
 
+
 def validate(to_predict, from_data, accuracy_score_functions, learn_args=None, test_args=None):
             if learn_args is None: learn_args = {}
             if test_args is None: test_args = {}
@@ -35,6 +36,7 @@ def validate(to_predict, from_data, accuracy_score_functions, learn_args=None, t
 
             return accuracy
 
+
 def cross_validate(to_predict, from_data, accuracy_score_functions, k=5, learn_args=None, test_args=None):
     '''
         Probably required a change to generate a split into `k` folds, then manually setting those folds as train/test/predict.
@@ -47,6 +49,7 @@ def cross_validate(to_predict, from_data, accuracy_score_functions, k=5, learn_a
     '''
     raise NotImplementedError('Cross validation is not implemented yet')
 
+
 def analyse_dataset(from_data, sample_settings=None):
     """
     Analyse the particular dataset being given
@@ -55,11 +58,13 @@ def analyse_dataset(from_data, sample_settings=None):
     from_ds = getDS(from_data)
     transaction_type = TRANSACTION_ANALYSE
 
-    sample_for_analysis, sample_for_training, disable_lightwood_transform_cache = _get_memory_optimizations(
-        from_ds.df)
-    sample_settings, sample_function = _prepare_sample_settings(sample_settings,
-                                                sample_for_analysis,
-                                                sample_for_training)
+    sample_for_analysis, sample_for_training, _ = _get_memory_optimizations(from_ds.df)
+
+    sample_settings, sample_function = _prepare_sample_settings(
+        sample_settings,
+        sample_for_analysis,
+        sample_for_training
+    )
 
     heavy_transaction_metadata = dict(
         name=None,
@@ -92,8 +97,9 @@ def analyse_dataset(from_data, sample_settings=None):
         logger=log
     )
 
-    md = get_model_data(lmd=tx.lmd)
-    return md
+    tx.run()
+
+    return get_model_data(lmd=tx.lmd)
 
 
 def export_storage(mindsdb_storage_dir='mindsdb_storage'):
@@ -103,8 +109,12 @@ def export_storage(mindsdb_storage_dir='mindsdb_storage'):
     :param mindsdb_storage_dir: this is the full_path where you want to store a mind to, it will be a zip file
     :return: bool (True/False) True if mind was exported successfully
     """
-    shutil.make_archive(base_name=mindsdb_storage_dir, format='zip',
-                        root_dir=CONFIG.MINDSDB_STORAGE_PATH)
+    shutil.make_archive(
+        base_name=mindsdb_storage_dir,
+        format='zip',
+        root_dir=CONFIG.MINDSDB_STORAGE_PATH
+    )
+
     print(f'Exported mindsdb storage to {mindsdb_storage_dir}.zip')
 
 
