@@ -64,7 +64,7 @@ def analyze_sentences(data):
     nr_words_dist = defaultdict(int)
     nltk.download('stopwords')
     stop_words = set(stopwords.words('english'))
-    for text in data:
+    for text in map(str, data):
         text = text.lower()
         tokens = tokenize_text(text)
         tokens_no_stop = [x for x in tokens if x not in stop_words]
@@ -75,11 +75,18 @@ def analyze_sentences(data):
 
     return nr_words, dict(word_dist), dict(nr_words_dist)
 
-def shrink_word_dist(word_dist):
-    tiny_word_dist = dict(sorted(word_dist.items(), key=lambda x: x[1], reverse=True)[:min(50,len(word_dist))])
+
+def shrink_word_dist(word_dist, limit=50):
+    tiny_word_dist = dict(
+        sorted(
+            word_dist.items(),
+            key=lambda x: x[1]
+        )[:limit]
+    )
     tiny_word_dist['other words'] = sum(word_dist.values()) - sum(tiny_word_dist.values())
     return tiny_word_dist
 
+
 def word_tokenize(string):
     sep_tag = '{#SEP#}'
     for separator in WORD_SEPARATORS:
